from enum import IntEnum

SLEEP_TIME = 0.01

class DestType:
    UNITY = "unity"
    WEB_IDE = "web-ide"


# 键盘按键
class KeyType(IntEnum):
    SPACE = 32
    DIGIT_0 = 48
    DIGIT_1 = 49
    DIGIT_2 = 50
    DIGIT_3 = 51
    DIGIT_4 = 52
    DIGIT_5 = 53
    DIGIT_6 = 54
    DIGIT_7 = 55
    DIGIT_8 = 56
    DIGIT_9 = 57
    A = 97
    B = 98
    C = 99
    D = 100
    E = 101
    F = 102
    G = 103
    H = 104
    I = 105
    J = 106
    K = 107
    L = 108
    M = 109
    N = 110
    O = 111
    P = 112
    Q = 113
    R = 114
    S = 115
    T = 116
    U = 117
    V = 118
    W = 119
    X = 120
    Y = 121
    Z = 122
    NUM_0 = 256
    NUM_1 = 257
    NUM_2 = 258
    NUM_3 = 259
    NUM_4 = 260
    NUM_5 = 261
    NUM_6 = 262
    NUM_7 = 263
    NUM_8 = 264
    NUM_9 = 265
    ARROW_UP = 273
    ARROW_DOWN = 274
    ARROW_RIGHT = 275
    ARROW_LEFT = 276
    SHIFT_RIGHT = 303
    SHIFT_LEFT = 304
    CONTROL_RIGHT = 305
    CONTROL_LEFT = 306
    MOUSE_LEFT = 323
    MOUSE_RIGHT = 324
    MOUSE_MIDDLE = 325


# 鼠标按键动作
class KeyActiveType(IntEnum):
    KEY_DOWN = 2
    KEY_UP = 4

class KeyPress(IntEnum):
    KEY_PRESS = 1

# 挂点
class HangPointType(IntEnum):
    BOTTOM = 1
    CAMERA = 2
    LEFT_FRONT_WHEEL = 3
    RIGHT_FRONT_WHEEL = 4
    LEFT_HAND = 10
    RIGHT_HAND = 11
    ITEM_HANGING_POINT = 100
    CAMERA_FOLLOW_POINT = 1000
    TOP = 2000
    USER_DEFINE = 0


# 角色动作
class Actions:
    PICK = "Pick"
    PLACE = "Place"
    LAUGH = "Laugh"
    HAPPY = "Happy"
    THINK = "Think"
    CONFUSE = "Confuse"
    SAD = "Sad"
    TALK = "Talk"
    GREET = "Greet"
    NO = "No"
    YES = "Yes"
    LOOKAROUND = "LookAround"
    APOLOGIZE = "Apologize"
    APPLAUD = "Applaud"
    BOW = "Bow"
    ANGRY = "Angry"
    FAINT = "Faint"
    ARMRESET = "ArmReset"
    DOWNPICK = "DownPick"
    UPPICK = "UpPick"
    REPAIR = "Repair"
    STANDGUARD = "StandGuard"


# 坐标类型 本地坐标或世界坐标
class AxisType(IntEnum):
    LOCAL = 1
    WORLD = 0


# 说话语气
class Tone:
    BOBO_ANGRY = "img_bobo_angry.png"
    BOBO_EXPRESSION = "img_bobo_expression.png"
    BOBO_SADNESS = "img_bobo_sadness.png"
    BOBO_SHY = "img_bobo_shy.png"
    BOBO_SMILE = "img_bobo_smile.png"
    BOBO_SURPRISE = "img_bobo_surprise.png"
    UU_ANGRY = "img_UU_angry.png"
    UU_EXPRESSION = "img_UU_expression.png"
    UU_SADNESS = "img_UU_sadness.png"
    UU_SHY = "img_UU_shy.png"
    UU_SMILE = "img_UU_smile.png"
    UU_SURPRISE = "img_UU_surprise.png"
    X_ANGRY = "img_x_angry.png"
    X_EXPRESSION = "img_x_expression.png"
    X_SADNESS = "img_x_sadness.png"
    X_SHY = "img_x_shy.png"
    X_SMILE = "img_x_smile.png"
    X_SURPRISE = "img_x_surprise.png"
    Y_ANGRY = "img_y_angry.png"
    Y_SADNESS = "img_y_expression.png"
    Y_SADNESS = "img_y_sadness.png"
    Y_SHY = "img_y_shy.png"
    Y_SMILE = "img_y_smile.png"
    Y_SURPRISE = "img_y_surprise.png"


# 音量大小
class Volume:
    LARGE = "large"
    MEDIUM = "medium"
    SMALL = "small"


# 立绘对话选项
class OptionName:
    OPTION01 = "option1"
    OPTION02 = "option2"
    OPTION03 = "option3"


# 提示面板展示内容
class ResultType:
    SUCCESS = "success"
    FAIL = "failure"
    START = "start"


# Toast提示位置
class ToastPosition:
    TOP = "top"
    BOTTOM = "bottom"
    MIDDLE = "middle"


# Toast提示状态
class ToastState:
    DYNAMIC = "dynamic"
    STATIC = "static"


# 手势方向
class Direction:
    LEFT = "left"
    RIGHT = "right"
    UP = "up"
    DOWN = "down"


# 滤镜类型
class FilterStyle(IntEnum):
    FOG = 1


# 对象的动作
class ActionType(IntEnum):
    ENTER = 1
    LEAVE = -1



from typing import Any, Union, Callable, List, Tuple, Set, Dict, get_origin, get_args, Generic, TypeVar, _GenericAlias,Literal
from functools import wraps
from crealand._utils import _logger_setup
import inspect
from inspect import signature
_logger = _logger_setup.setup_logger()
# 限制参数值范围
def value_range(
    val: float,
    min: float = float("-inf"),
    max: float = float("-inf"),
):
    if val < min:
        val = min
    elif val > max:
        val = max
    return val

def Handle_point(value: tuple):
    result = {"id": value[0]}
    if len(value) ==2:
        result["name"] = value[1]
    return result

def check_type(func):
    @wraps(func)
    def inner(*args, **kwargs):
        # 获取原始函数
        original_func = func
        is_classmethod = isinstance(original_func, classmethod)
        if isinstance(original_func, (staticmethod, classmethod)):
            original_func = original_func.__func__
        
        # 获取函数的类型注解
        annotations = original_func.__annotations__

        # 获取函数的签名信息
        signature = inspect.signature(original_func)
        parameters = signature.parameters

        # 获取函数的参数名称
        arg_names = list(parameters.keys())

        # 处理 classmethod 的 cls 参数
        cls_arg=None
        if is_classmethod:
            if args:
                cls_arg=args[0]
                arg_names = arg_names[1:] 
            elif 'cls' in kwargs:
                cls_arg=kwargs.pop('cls')
                arg_names = arg_names[1:]
            else:
                raise TypeError(f"{original_func.__name__}() missing 1 required positional argument: 'cls'")

        # 检查位置参数数量
        if len(args) > len(arg_names):
            raise TypeError(f"{original_func.__name__}() takes {len(arg_names)} positional arguments but {len(args)} were given")
        
        # 检查位置参数
        for i, arg in enumerate(args):
            if i < len(arg_names):
                param_name = arg_names[i]
                param = parameters[param_name]
                if param_name in annotations:
                    annotation = annotations[param_name]
                    if not is_instance_of(arg, annotation):
                        raise TypeError(f"{original_func.__name__}() argument type wrong: {param_name} must be of type {annotation}, but got {type(arg)}")
        
        # 检查关键字参数
        for key, value in kwargs.items():
            if key in parameters:
                param = parameters[key]
                if param.default == inspect.Parameter.empty and key in annotations:
                    annotation = annotations[key]
                    if not is_instance_of(value, annotation):
                        raise TypeError(f"{original_func.__name__}() argument type wrong: {key}  must be of type {annotation}, but got {type(value)}")

        # 检查是否有未传递的必需参数
        provided_args = list(args) + list(kwargs.keys())
        required_params = [name for name in arg_names if name not in provided_args and parameters[name].default == inspect.Parameter.empty]
        if len(provided_args) <len(required_params):
            raise TypeError(f"{original_func.__name__}() missing {len(required_params) - len(provided_args)} required positional arguments: {', '.join(required_params[len(provided_args):])}")
        # if required_params:
        #     raise TypeError(f"{original_func.__name__}() missing {len(required_params)} required positional arguments: {', '.join(required_params)}")
        
        if is_classmethod:
            ar=arg_names
            a=args
            k=kwargs
            return original_func(cls_arg, *args, **kwargs)
        else:
            return original_func(*args, **kwargs)
    
    def is_instance_of(value, annotation):
        if annotation is Any:
            return True
        if get_origin(annotation) is Union:
            return any(is_instance_of(value, t) for t in get_args(annotation))
        elif annotation is Callable:
            return callable(value)
        elif get_origin(annotation) in (List, Tuple, Set, Dict,Literal):
            if get_origin(annotation) is Literal:
                return value in get_args(annotation)
            if not isinstance(value, get_origin(annotation)):
                return False
            if not get_args(annotation):  # 如果没有指定元素类型，直接返回 True
                return True
            if get_origin(annotation) is dict:
                key_type, value_type = get_args(annotation)
                return all(is_instance_of(k, key_type) and is_instance_of(v, value_type) for k, v in value.items())
            else:
                element_type = get_args(annotation)[0]
                return all(is_instance_of(item, element_type) for item in value)
        elif isinstance(annotation, type):
            return isinstance(value, annotation)
        elif isinstance(annotation, _GenericAlias):
            origin = get_origin(annotation)
            args = get_args(annotation)
            if origin is None:
                return False
            if not isinstance(value, origin):
                return False
            if args:
                if origin is dict:
                    key_type, value_type = args
                    return all(is_instance_of(k, key_type) and is_instance_of(v, value_type) for k, v in value.items())
                elif origin in (list, tuple, set):
                    element_type = args[0]
                    if get_origin(annotation) is Union:
                        return any(is_instance_of(item, element_type) for item in value)
                    else:
                        return all(is_instance_of(item, element_type) for item in value)
            return True
        return False
    
    return inner

def raise_error(func,err,data):
    if err > 0:
        raise Exception(f"Error occurred:func {func} {err}, {data}")
    else:
        _logger.info(f"Error occurred:func {func} {err}, {data}")


# 正式版本号
__version__ = '1.0.44'
# 测试版本号
# __version__ = '1.0.9'

from ._utils._logger_setup import (
    enable_user_log, 
    disable_user_log, 
    setup_logger
)

def __initialize():
    import os
    from ._utils._logger_setup import set_log_dir

    server_url = os.getenv('ENV_CREALAND_UCODELINK_SERVER_URL')
    if not server_url or server_url == '':
        return
    session_id = os.getenv('ENV_CREALAND_SESSIONID')
    task_id = os.getenv('ENV_CREALAND_TASKID')
    enable_log = os.getenv('ENV_CREALAND_SDK_ENABLE_LOG')
    log_dir = os.getenv('ENV_CREALAND_SDK_DIR')

    if enable_log == 'True':
        enable_user_log()
    if log_dir:
        set_log_dir(log_dir)

    logger = setup_logger()
    logger.info(f'server_url: {server_url}, session_id: {session_id}, \
task_id: {task_id}, enable_log: {enable_log}, log_dir: {log_dir}')
    try:
        import time
        import threading
        from ._core.bridge._interface import call_api
        from ._core.websocket.websocket_client import ws_connect, get_ws_client, init_callback
        threading.Thread(target=ws_connect, args=(server_url, session_id, task_id, )).start()
        while True:
            time.sleep(0.1)
            ws_client = get_ws_client()
            if (ws_client and ws_client.websocket and 
                ws_client.connected and ws_client.session_id):
                break
        init_callback()
        call_api('web-ide', "api.prepareCompleted", [{}])
    except Exception as e:
        logger.error(f'An error occurred: {e}')

__initialize()

from ._apis import (
    ai as AI, 
    detect as Detect, 
    event as Event, 
    interactive as Interactive, 
    object as Object
)

from ._apis._constants import (
    KeyType, 
    KeyActiveType, 
    HangPointType, 
    Actions, 
    AxisType, 
    Tone, 
    Volume, 
    OptionName, 
    ResultType, 
    ToastPosition, 
    ToastState, 
    Direction,  
    FilterStyle, 
    ActionType
)

from ._apis.interactive import (
    Dialogue, 
    HelpPanel, 
    TaskPanel, 
    Speak, 
    Interactive
)

from ._apis.media import (
    Sound, 
    Video, 
    Image
)

from ._apis.sensor import (
    Ultrasonic, 
    Auditory, 
    Visual, 
    Temperature, 
    Humidity, 
    Gravity
)





from crealand._core.bridge import _interface
from crealand._apis import _subscribe_event
from crealand._utils import _utils
from typing import Any, Union

_decibel_val = 0

# 分贝值
def get_decibel_value():
    global _decibel_val
    return _decibel_val

# 开始识别
def start_decibel_recognition():

    def cb_wrapper(err,data):
        if err is None:
           global _decibel_val
           _decibel_val = data['data']
        else:
           _utils.raise_error('start_decibel_recognition',err,data)

    _subscribe_event.onSensorSoundEvent('==','',cb_wrapper)

# 结束识别
def stop_decibel_recognition():
    _interface.call_api_async('web-ide', 'api.openDecibelDetectionPage', [{'type':'end'}])

@_utils.check_type
def onEventDecibel(decibel_value:Union[int,float],cb:Any):
    decibel_value = max(0, min(decibel_value, 150))
    def cb_wrapper(err,data):
        if err is None:
           global _decibel_val
           _decibel_val = data['data']
           cb()
        else:
           _utils.raise_error('onEventDecibel',err,data)


    _subscribe_event.onSensorSoundEvent('>',decibel_value,cb_wrapper)


# 获取虚拟相机
@_utils.check_type
def virtual_camera(runtime_id: Union[int,float],status: bool=True):
    _interface.call_api('unity', 'unity.camera.openVirsual', [runtime_id,status])



from crealand._core.bridge import _interface
from crealand._apis import _constants
from typing import Union
from crealand._utils import _utils

class Dialogue:
    # 立绘对话 获取选项
    option_value = ''
    @staticmethod
    def get_option_value():
        return Dialogue.option_value

    # 立绘对话 初始化
    @staticmethod
    def init():
        _interface.call_api('web-ide', "api.prepareDialogBoard", [{}])
        

    # 立绘对话 显示
    @_utils.check_type
    @staticmethod
    def set_dialogue(
        obj_name: str,
        content: str,
        res_id: str='',
        volume: str = _constants.Volume.MEDIUM,
    ):
        _interface.call_api(
            'web-ide',
            "api.showDialog",
            [
                {
                    "speaker": obj_name,
                    "type": volume,
                    "txt": content,
                    "voiceId": '',
                    "pythonOssId":res_id,
                }
            ],
        )
    
    @_utils.check_type
    @staticmethod
    def set_dialogue_tone(
        obj_name: str,
        content: str,
        res_id: str='',
        tone: str='',
        volume: str = _constants.Volume.MEDIUM,
    ):
        _interface.call_api(
            'web-ide',
            "api.showDialog",
            [
                {
                    "speaker": obj_name,
                    "type": volume,
                    "txt": content,
                    "voiceId": tone,
                    "pythonOssId":res_id,
                }
            ],
        )

    # 立绘对话 设置选项
    @_utils.check_type
    @staticmethod
    def set_option( content: str, opt_name: str = _constants.OptionName.OPTION01):
        options = {}
        options[opt_name] = content
        _interface.call_api(
            'web-ide',
            "api.setDialogOptions",
            [{"options": options}],
        )

    # 立绘对话选项 显示
    @_utils.check_type
    @staticmethod
    def set_option_show(is_show: bool = True):
        Dialogue.option_value = _interface.call_api('web-ide', "api.toggleDialogOptions", [{"show": is_show}])
    #  立绘对话 显示
    @_utils.check_type
    @staticmethod
    def show(is_show: bool = True):
        if not is_show:
           Dialogue.option_value = ''
            
        _interface.call_api('web-ide', "api.toggleDialogBoard", [{"show": is_show}])


class HelpPanel:
    # 帮助面板 初始化
    @staticmethod
    def init():
        _interface.call_api('web-ide', "api.prepareHelpboard", [{}])

    # 帮助面板 设置标题
    @_utils.check_type
    @staticmethod
    def set_tips(title: str, res_id: str=''):
        _interface.call_api(
            'web-ide',
            "api.addHelpItem",
            [
                {
                    "title": title,
                    "pythonOssId": res_id,
                }
            ],
        )

    # 帮助面板 显示
    @_utils.check_type
    @staticmethod
    def show(is_show: bool = True):
        _interface.call_api(
            'web-ide',
            "api.toggleHelpboard",
            [
                {
                    "show": is_show,
                }
            ],
        )


class TaskPanel:

    # 任务面板 设置标题
    @_utils.check_type
    @staticmethod
    def set_task(title: str, nickname: str):
        _interface.call_api(
            'web-ide',
            "api.createTaskboard",
            [
                {
                    "title": title,
                    "alias": nickname,
                }
            ],
        )

    # 任务面板 设置任务项
    @_utils.check_type
    @staticmethod
    def set_task_progress(
        task_name: str, subtasks_content: str, completed_tasks: int, total_tasks: int
    ):
        _interface.call_api(
            'web-ide',
            "api.setTaskboard",
            [
                {
                    "alias": task_name,
                    "taskName": subtasks_content,
                    "process": [max(0, completed_tasks), max(1, total_tasks)],
                }
            ],
        )

    # 任务面板 显示
    @_utils.check_type
    @staticmethod
    def set_task_show(task_name: str, is_show: bool = True):
        _interface.call_api(
            'web-ide',
            "api.toggleTaskboard",
            [{"alias": task_name, "show": is_show}],
        )


class Speak:
    # 说
    @_utils.check_type
    @staticmethod
    def text( runtime_id: Union[int,float], content: str, time: int = 2):
        _interface.call_api_async(
            'unity', "unity.actor.speak", [runtime_id, content, time]
        )

    # 说-img
    @_utils.check_type
    @staticmethod
    def image(runtime_id: Union[int,float], res_id: str, time: int = 2):
        _interface.call_api_async(
           'unity', "unity.actor.speakImage", [runtime_id, res_id, time]
        )


class Interactive:
    # 提示面板显示
    @_utils.check_type
    @staticmethod
    def set_tip_show(option: str = _constants.ResultType.START):
        _interface.call_api(
            'web-ide',
            "api.showTipboardResult",
            [
                {
                    "result": option,
                }
            ],
        )

    # 提示面板显示
    @_utils.check_type
    @staticmethod
    def toast(
        content: str,
        position: str = _constants.ToastPosition.TOP,
        state: str = _constants.ToastState.DYNAMIC,
    ):
        _interface.call_api_async(
            'web-ide',
            "api.toast",
            [
                {
                    "position": position,
                    "mode": state,
                    "txt": content,
                }
            ],
            
        )



from crealand._core.bridge import _interface
from crealand._apis import _subscribe_event, _constants
from crealand._utils import _utils
from typing import Any,Union
import time

#收到广播事件
@_utils.check_type
def onBroadcastEvent(info:str,cb:Any):
    _subscribe_event.onBroadcastEvent(info,cb)

#发送广播事件
@_utils.check_type
def send_broadcast(info:str):
    _subscribe_event.sendBroadcast(info)

#对象进入/离开判定区域事件
@_utils.check_type
def onAreaObjectEvent(runtime_id:Union[int,float],action:int,area_id:Union[int,float],cb:Any):
    def cb_wrapper(err,data):
        if err is None:
            cb()
        else:
            _utils.raise_error('onAreaObjectEvent',err,data)
    _subscribe_event.onAreaObjectEvent(runtime_id,action,area_id,cb_wrapper)

#分类进入/离开判定区域事件
@_utils.check_type
def onAreaClassEvent(config_id:str,action:int,area_id:Union[int,float],cb:Any):
    def cb_wrapper(err,data):
        if err is None:
            cb()
        else:
            _utils.raise_error('onAreaClassEvent',err,data)
    _subscribe_event.onAreaClassEvent(config_id,action,area_id,cb_wrapper)

# 验证按键是否按下
@_utils.check_type
def keypress_state(button:int):
    time.sleep(_constants.SLEEP_TIME)
    result = _interface.call_api('unity',"unity.input.verifyKeyCodeState",[button, _constants.KeyPress.KEY_PRESS])
    return result

# 鼠标键盘事件
@_utils.check_type
def onKeyEvent(action:int,button:int,cb:Any):
    def cb_wrapper(err,data):
        if err is None:
            cb()
        else:
            _utils.raise_error('onKeyEvent',err,data)
    _subscribe_event.onKeyEvent(action,button,cb_wrapper)



from crealand._core.bridge import _interface
from crealand._apis import _constants
from crealand._utils import _utils
from typing import List, Union, Literal

# 信息
class Info:

    # 别名对象id
    @_utils.check_type
    @staticmethod
    def get_alias_id(
        nickname: str ,
    ):
        result = _interface.call_api(
            'unity', "unity.alias.getAlias", [nickname]
        )
        return result

    # 获取configID的对象id
    @_utils.check_type
    @staticmethod
    def get_object_id(runtime_id:Union[int,float]) -> int:
        result = _interface.call_api(
            'unity',
            "unity.actor.getConfigID",
           [runtime_id] ,
        )
        return result

    # 获取对象的空间坐标
    @_utils.check_type
    @staticmethod
    def get_object_coordinates(runtime_id: Union[int,float]) :
        result = _interface.call_api(
            'unity', "unity.actor.getCoordinate", [runtime_id]
        )
        return result

    # 获取判定区域中的对象id
    @_utils.check_type
    @staticmethod
    def get_id_in_area(area_id: Union[int,float], config_ids: List[str]) :
        result = _interface.call_api(
            'unity',
            "unity.editableTrigger.getContentRuntimeIds",
            [area_id, config_ids],
        )
        return result

    # 获取空间坐标某个轴的值
    @_utils.check_type
    @staticmethod
    def get_spatial_coordinates(coordinate: List[Union[int,float]], axis: Literal['X', 'Y','Z']='X') :
        AXIS = {"X": 0, "Y": 1, "Z": 2}
        return coordinate[AXIS[axis]]

    # 获取对象的运动方向向量
    @_utils.check_type
    @staticmethod
    def get_motion_vector(runtime_id: Union[int,float]) :
        result = _interface.call_api(
            'unity', "unity.character.getMoveDirection", [runtime_id]
        )
        return result


class Camera:

    # 获取相机ID
    @_utils.check_type
    @staticmethod
    def get_default_id():
        return _interface.call_api(
            'unity', "unity.camera.getDefaultID",[]
        )

    # 获取空间坐标
    @_utils.check_type
    @staticmethod
    def get_object_coordinates(runtime_id: Union[int,float]) :
        result = _interface.call_api(
            'unity', "unity.actor.getCoordinate", [runtime_id]
        )
        return result

    # 相机移动
    @_utils.check_type
    @staticmethod
    def move_to(time: Union[int,float], coordinate: List[Union[int,float]], block: bool = False):
        new_time = max(0, time)
        _interface.call_api(
            'unity',
            "unity.camera.moveTo",
            [Camera.get_default_id(), new_time, coordinate, block ],
        )

    # 调整FOV
    @_utils.check_type
    @staticmethod
    def adjust_FOV(time: Union[int,float] = 1, fov: Union[int,float] = 80):
        new_time = max(0, time)
        new_fov = max(60, min(fov, 120))
        _interface.call_api_async(
            'unity',
            "unity.camera.adjustFOV",
            [Camera.get_default_id(), new_time, new_fov],
        )

    # 相机锁定朝向并移动
    @_utils.check_type
    @staticmethod
    def move_while_looking(
        coordinate_1: List[Union[int,float]],
        time: Union[int,float] = 1,
        coordinate_2: List[Union[int,float]] = [0, 0, 1],
        block: bool = False,
    ):
        new_time = max(0, time)
        _interface.call_api_async(
            'unity',
            "unity.camera.moveWhileLooking",
            [Camera.get_default_id(), new_time, coordinate_2, coordinate_1, block],
        )

    # 获取相机坐标
    @_utils.check_type
    @staticmethod
    def get_camera_coordinate() -> List[Union[int,float]]:
        result = Camera.get_object_coordinates(Camera.get_default_id())
        return result

    # 相机朝向
    @_utils.check_type
    @staticmethod
    def look_at(coordinate: List[Union[int,float]]):
        _interface.call_api_async(
            'unity',
            "unity.camera.lookAt",
            [Camera.get_default_id(), coordinate],
        )

    # 相机跟随
    @_utils.check_type
    @staticmethod
    def follow_target(runtime_id: int, distance: Union[int,float] = 10, is_rotate: bool = True):
        _interface.call_api_async(
            'unity',
            "unity.camera.followTarget",
            [Camera.get_default_id(), runtime_id, distance, is_rotate],
        )

    # 相机结束跟随
    @_utils.check_type
    @staticmethod
    def end_follow_target():
        _interface.call_api_async(
            'unity',
            "unity.camera.stopFollowing",
            [
                Camera.get_default_id(),
            ],
        )

    # 相机 滤镜
    @_utils.check_type
    @staticmethod
    def filters(filter_name: int = _constants.FilterStyle.FOG, state: bool = True):
 
        _interface.call_api_async(
            'unity',
            "unity.camera.openEffect",
            [Camera.get_default_id(), filter_name, state],
        )


class Motion:
    # 创建对象
    @_utils.check_type
    @staticmethod
    def create_object_coordinate(config_id: str, coordinate: List[Union[int,float]]):
        result = _interface.call_api(
            'unity',
            "unity.actor.createObject",
            [config_id, coordinate],
        )
        return result

    # 测距
    @_utils.check_type
    @staticmethod
    def ray_ranging(runtime_id: int, attachment_id: int = (_constants.HangPointType.LEFT_FRONT_WHEEL,)):
        
        result = _interface.call_api(
            'unity',
            "unity.actor.rayRanging",
            [runtime_id, _utils.Handle_point(attachment_id), 20],
        )
        return result

    # 移动
    @_utils.check_type
    @staticmethod
    def move_to(runtime_id: int, coordinate: List[Union[int,float]] = [0, 0, 1]):
        _interface.call_api(
            'unity',
            "unity.actor.setObjectPosition",
            [
                runtime_id,
                coordinate,
            ],
        )

    # 朝向
    @_utils.check_type
    @staticmethod
    def face_towards(runtime_id: Union[int,float], coordinate: List[Union[int,float]] = [0, 0, 1]):
        _interface.call_api(
            'unity',
            "unity.actor.setObjectTowardPosition",
            [
                runtime_id,
                coordinate,
            ],
        )

    # 前进
    @_utils.check_type
    @staticmethod
    def move_forward(
        runtime_id: Union[int,float], speed: Union[int,float] =1, distance: Union[int,float] =3, block: bool = False
    ):
        new_speed = max(1, min(speed, 5))
        _interface.call_api(
            'unity',
            "unity.actor.moveForwardByDistance",
            [
                runtime_id,
                distance,
                abs(distance/new_speed),
            block ,
            ],
        )

    # 对象旋转
    @_utils.check_type
    @staticmethod
    def rotate(runtime_id: Union[int,float], time: Union[int,float]=1, angle: Union[int,float]=90, block: bool = False):
        new_time = max(time, 0)
        _interface.call_api(
            'unity',
            "unity.character.rotateUpAxisByAngle",
            [
                runtime_id,
                angle,
                new_time,
                block 
            ],
        )

    # 云台旋转 & 机械臂旋转
    @_utils.check_type
    @staticmethod
    def ptz(runtime_id: Union[int,float], angle: Union[int,float]=90, block: bool = False):
        _interface.call_api(
            'unity',
            "unity.actor.rotatePTZUpAxisByAngle",
            [runtime_id, angle, abs(angle) / 30, block],
        )

    # 播放动作
    @_utils.check_type
    @staticmethod
    def action(runtime_id: Union[int,float], action: str, block: bool = False):
        _interface.call_api(
            'unity',
            "unity.actor.playAnimation",
            [runtime_id, action, block ],
        )

    # # 将对象吸附到挂点
    @_utils.check_type
    @staticmethod
    def attach_to(absorbed_runtime_id: Union[int,float], absorb_runtime_id: Union[int,float], attachment_id: tuple):
       
        _interface.call_api(
            'unity',
            "unity.actor.attach",
            [absorbed_runtime_id, absorb_runtime_id,  _utils.Handle_point(attachment_id)],
        )

    # 绑定挂点
    @_utils.check_type
    @staticmethod
    def bind_to_object_point(
        runtime_id_1: Union[int,float],
        attachment_id_1: str,
        runtime_id_2: Union[int,float],
        attachment_id_2: str,
    ):
        _interface.call_api(
            'unity',
            "unity.actor.bindAnchor",
            [runtime_id_1, _utils.Handle_point(attachment_id_1), runtime_id_2, _utils.Handle_point(attachment_id_2)],
        )

    # 解除绑定
    @_utils.check_type
    @staticmethod
    def detach(runtime_id: Union[int,float]):
        _interface.call_api(
            'unity',
            "unity.actor.detach",
            [
                runtime_id,
            ],
        )

    # 向画面空间前进
    @_utils.check_type
    @staticmethod
    def move_towards_screen_space(
        runtime_id: Union[int,float], speed: Union[int,float] = 1, direction: List[Union[int,float]] = [0, 0, 1]
    ):
        new_speed = max(1, min(speed, 5))
        _interface.call_api(
            'unity',
            "unity.actor.moveByVelocity",
            [
                runtime_id,
                new_speed,
                2,
                direction,
            ],
        )

    @_utils.check_type
    @staticmethod
    def get_motion_vector(runtime_id: Union[int,float]):
        result = _interface.call_api('unity', "unity.character.getMoveDirection", [runtime_id])
        return result

    # 旋转运动方向向量
    @_utils.check_type
    @staticmethod
    def rotate_to_direction(
        runtime_id: Union[int,float], angle: Union[int,float] = 0, direction: List[Union[int,float]] = [0, 0, 1]
    ):
        _interface.call_api(
            'unity',
            "unity.character.rotateUpAxisByDirection",
            [runtime_id, angle, direction,0],
        )

    # 停止运动
    @_utils.check_type
    @staticmethod
    def stop(runtime_id: Union[int,float]):
        _interface.call_api_async(
            'unity',
            "unity.character.stop",
            [runtime_id],
        )

    # 设置别名
    @_utils.check_type
    @staticmethod
    def create_object(
        config_id: str,
        nickname: str,
        coordinate: List[Union[int,float]] = [0, 0, 1],
    ):
        _interface.call_api_async(
            'unity',
            "unity.alias.setAlias",
            [
                nickname,
                Motion.create_object_coordinate(config_id, coordinate),
            ],
        )

    # 销毁对象
    @_utils.check_type
    @staticmethod
    def destroy(runtime_id: Union[int,float]):
        _interface.call_api(
            'unity',
            "unity.alias.destoryObject",
            [
                runtime_id,
            ],
        )

    # 上升
    @_utils.check_type
    @staticmethod
    def rise(
        runtime_id: Union[int,float], speed: Union[int,float] = 3, height: Union[int,float] = 10, block: bool = False
    ):
        new_speed = max(1, min(speed, 5))
        _interface.call_api(
            'unity',
            "unity.character.moveUpByDistance",
            [runtime_id, height, abs(height/new_speed), block],
        )
    # 降落
    @_utils.check_type
    @staticmethod
    def landing(
        runtime_id: Union[int,float] 
    ):
        _interface.call_api(
            'unity',
            "unity.character.land",
            [runtime_id, 3],
        )

    # 获取离自身距离的坐标
    @_utils.check_type
    @staticmethod
    def get_object_local_position(
        runtime_id: Union[int,float], coordinate: List[Union[int,float]] = [0, 0, 1], distance: Union[int,float] = 0
    ):
        result = _interface.call_api_async(
            'unity',
            "unity.actor.getObjectLocalPosition",
            [runtime_id, coordinate, distance],
        )
        return result

    # 移动到指定坐标
    @_utils.check_type
    @staticmethod
    def move_by_point(
        runtime_id: int, time: Union[int,float] = 1, coordinate: List[Union[int,float]] = [0, 0, 1], block: bool = False
    ):
        new_time = max(time, 0)
        _interface.call_api(
            'unity',
            "unity.actor.moveByPoint",
            [runtime_id, new_time, coordinate, block ],
        )

    # 绕坐标轴旋转
    @_utils.check_type
    @staticmethod
    def rotate_by_origin_and_axis(
        runtime_id: Union[int,float],
        time: Union[int,float] = 2,
        point_1: int=_constants.AxisType.LOCAL,
        coordinate_1: List[Union[int,float]] = [0, 0, 0],
        point_2: int=_constants.AxisType.LOCAL,
        coordinate_2: List[Union[int,float]]= [0, 0, 1],
        angle: Union[int,float]=90,
        block: bool = False,
    ):
        new_time = max(time, 0)
        _interface.call_api(
            'unity',
            "unity.actor.rotateByOringinAndAxis",
            [
                runtime_id,
                coordinate_1,
                point_1,
                coordinate_2,
                point_2,
                angle,
                new_time,
                block 
            ],
        )


class Property:
    # 新增自定义属性
    @_utils.check_type
    @staticmethod
    def add_attr(runtime_id: Union[int,float], attr_name: str, attr_value: str):
        _interface.call_api(
            'unity',
            "unity.actor.addCustomProp",
            [runtime_id, attr_name, attr_value],
        )

    # 删除自定义属性
    @_utils.check_type
    @staticmethod
    def del_attr(runtime_id: Union[int,float], attr_name: str):
        _interface.call_api(
            'unity',
            "unity.actor.delCustomProp",
            [runtime_id, attr_name],
        )

    # 修改自定义属性
    @_utils.check_type
    @staticmethod
    def set_attr(runtime_id: Union[int,float], attr_name: str, attr_value: str):
        _interface.call_api(
            'unity',
            "unity.actor.setCustomProp",
            [runtime_id, attr_name, attr_value],
        )

    # 获取自定义属性的值
    @_utils.check_type
    @staticmethod
    def get_value(runtime_id: Union[int,float], attr_name: str):
        result = _interface.call_api(
            'unity',
            "unity.actor.getCustomProp",
            [runtime_id, attr_name],
        )
        return result

    # 获取自定义属性组中某一项的值
    @_utils.check_type
    @staticmethod
    def get_value_by_idx(runtime_id: Union[int,float], index: int = 1):
        result= _interface.call_api(
            'unity',
            "unity.actor.getCustomPropValueByIdx",
            [runtime_id, index],
        )
        return result

    # 获取自定义属性组中某一项的名称
    @_utils.check_type
    @staticmethod
    def get_key_by_idx(runtime_id: Union[int,float], index: int = 1):
        result=_interface.call_api(
            'unity',
            "unity.actor.getCustomPropKeyByIdx",
            [runtime_id, index],
        )
        return result


class Show:
    # 3d文本-RGB
    @_utils.check_type
    @staticmethod
    def set_3D_text_status_rgb(runtime_id: Union[int,float], rgb: List[Union[int,float]] = [255, 255, 255], size: int = 30, text: str = '文本'):
        _interface.call_api(
            'unity',
            "unity.building.set3DTextStatus",
            [runtime_id, rgb, size, text],
        )



import asyncio
import uuid
from typing import Literal
from crealand._core.websocket.websocket_client import get_ws_client, get_event_loop, get_callback
from crealand._utils._logger_setup import setup_logger

logger = setup_logger()

# call API
def call_api(dest: Literal['web-ide', 'unity'], func_name: str, func_args: list):
    #logger.info(f'{dest}, {func_name}, {func_args}')
    bridge_msg = {
        'id': str(uuid.uuid4()),
        'func': func_name,
        'args': func_args if func_args else [],
        'callback': 'callbackName'
    }

    loop = get_event_loop()
    ws_client = get_ws_client()
    task = asyncio.run_coroutine_threadsafe(ws_client.sendToCrealandApiTransfer(dest, ws_client.session_id, bridge_msg), loop)
    result = task.result()
    return result

def call_api_async(dest: Literal['web-ide', 'unity'], func_name: str, func_args: list, callback=None):
    #logger.info(f'{dest}, {func_name}, {func_args}, {callback.__name__}')
    bridge_msg_id = str(uuid.uuid4())
    bridge_msg = {
        'id': bridge_msg_id,
        'func': func_name,
        'args': func_args if func_args else [],
        'callback': 'callbackName'
    }

    loop = get_event_loop()
    ws_client = get_ws_client()
    task = asyncio.run_coroutine_threadsafe(ws_client.sendToCrealandApiTransferAsync(dest, ws_client.session_id, bridge_msg), loop)
    result = task.result()    
    if result and callback:
        api_callback = get_callback()
        api_callback.registerCallback(bridge_msg_id, callback)

    return result



from crealand._core.bridge import _interface
from crealand._apis import _subscribe_event
from crealand._utils import _logger_setup, _utils
from typing import Any

_logger = _logger_setup.setup_logger()

# 人脸识别
# 机器学习
# 数字识别


class Figure:

    _digit=''
    # 打开手写数字识别教学页面
    @staticmethod
    def open_teach_page():
        _interface.call_api('web-ide','api.openDigitRecognitionTeachingPage',[{'name':''}])

    # 打开神经网络教学页面
    @staticmethod
    def open_NN_teach_page():
        _interface.call_api('web-ide','api.openNeuralNetworkTeachingPage',[{'type':''}])

    # 开始手写数字识别
    @staticmethod
    def start_digital_recognition():
        # 识别返回的结果需要设置保存
        # call_api_async('web-ide','api.digitRecognition',[{'type':'start'}])
        def on_result(err,data):
            if err is None:
                if data and data.get('data') and data['data'] != '':
                    Figure._digit = int(data['data'])
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIFigureEvent('',on_result)

    # 结束手写数字识别
    @staticmethod
    def stop_digital_recognition():
        Figure._digit =''
        _interface.call_api_async('web-ide','api.digitRecognition',[{'type':'end'}])

    # 数字识别结果
    @staticmethod
    def get_figure_value():
        return Figure._digit

    # 清除数字识别结果
    @staticmethod
    def clear_figure_value():
        Figure._digit =''
        _interface.call_api('web-ide','api.digitRecognitionClear',[{'type':''}])

    @_utils.check_type
    @staticmethod
    def onAIFigureEvent(number:int,cb:Any):
        # 识别返回的结果需要设置保存
        # call_api_async('web-ide','api.digitRecognition',[{'type':'start'}])
        def on_result(err,data):
            if err is None:
                if data and data.get('data') and data['data'] != '':
                    Figure._digit = int(data['data'])
                    cb()
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIFigureEvent(number,on_result)


# 手势识别
class Gesture:
    _direction=''
    # 开始手势识别 并等待结束
    @staticmethod
    def start_gesture_recognition():
        def on_result(err,data):
            if err is None:
                if data and data.get('data') and data['data'] != '':
                    Gesture._direction = data['data']
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIGestureEvent('',on_result)

    # 结束手势识别
    @staticmethod
    def stop_gesture_recognition():
        Gesture._direction=''
        _interface.call_api_async('web-ide','api.gestureRecognition',[{'type':'end'}])

    # 当前手势识别结果为
    @staticmethod
    def get_gesture_value():
        return Gesture._direction

    # 帽子积木块
    @_utils.check_type
    @staticmethod
    def onAIGestureEvent(direction:str,cb:Any):
        def on_result(err,data):
            if err is None:
                if data and data.get('data') and data['data'] != '':
                    Gesture._direction = data['data']
                    cb()
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIGestureEvent(direction,on_result)

# 语音识别
class Voice:
    _text=''
    # 开始语音识别
    @staticmethod
    def start_voice_recognition():
        def on_result(err,data):
            if err is None:
                if data and data.get('data') and data['data'] != '':
                    Voice._text = data['data']
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIAsrEvent('',on_result)


    # 结束识别
    @staticmethod
    def stop_voice_recognition():
        Voice._text =''
        _interface.call_api('web-ide','api.openVoiceRecognition',[{'type':'end'}])

    # 语音识别结果
    @staticmethod
    def get_voice_value():
        return Voice._text


    # 打开语音识别教学页面
    @staticmethod
    def open_voice_teach_page():
        _interface.call_api('web-ide','api.openVoiceRecognitionTeachingPage',[{'name':''}])

    # 帽子积木块
    @_utils.check_type
    @staticmethod
    def onAIAsrEvent(text:str,cb:Any):
        def on_result(err,data):
            if err is None:
                if data and data.get('data') and data['data'] != '':
                    Voice._text = data['data']
                    cb()
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIAsrEvent(text,on_result)





import time
from crealand._core.bridge import _interface
from crealand._apis import _subscribe_event, _constants
from typing import Any, Union
from crealand._utils import _utils

# 超声波传感器

class Ultrasonic:
    _sensors = {}

    # 前端处理传感器信息绑定
    @_utils.check_type
    @staticmethod
    def add_sensor(sensor: str, runtime_id: Union[int,float], attachment_id: tuple) ->None:
        attach=_utils.Handle_point(attachment_id)
        Ultrasonic._sensors[sensor] = (runtime_id, attach)

    @_utils.check_type
    @staticmethod
    def get_sensor(sensor: str) -> list:
        if sensor in Ultrasonic._sensors:
            return Ultrasonic._sensors[sensor]
        else:
            raise KeyError(f"Sensor '{sensor}' not found")

    @_utils.check_type
    @staticmethod
    def onSensorUltrasonicEvent(sensor: str, compare: str, distance: Union[int,float],cb:Any):
        sensor_info = Ultrasonic.get_sensor(sensor) 
        attachment_id = sensor_info[1]
        def cb_wrapper(err,data):
            if err is None:
                cb()
            else:
                _utils.raise_error('onSensorUltrasonicEvent',err,data)

        _subscribe_event.onSensorUltrasonicEvent(sensor_info[0],attachment_id,compare,distance,cb_wrapper)

    @_utils.check_type
    @staticmethod
    def get_obstacle_distance(sensor: str)->Union[int,float]:
        time.sleep(_constants.SLEEP_TIME)
        sensor_info = Ultrasonic.get_sensor(sensor) 
        length = _interface.call_api('unity', 'unity.sensor.ultrasonicRanging', [sensor_info[0], sensor_info[1]])
        return length

class Auditory:

    _decibel_val=0

    @_utils.check_type
    # 获取声音强度
    @staticmethod
    def get_decibel_value():
        return Auditory._decibel_val

    @_utils.check_type
    @staticmethod
    def onSensorSoundEvent(compare:str,decibel_value:Union[int,float],cb:Any):
        decibel_value = max(0, min(decibel_value, 150))
        def cb_wrapper(err,data):
            if err is None:
                if data and data.get('data') and data['data'] != '':
                    Auditory._decibel_val = data['data']
                    cb()
            else:
                _utils.raise_error('onSensorSoundEvent',err,data)

        _subscribe_event.onSensorSoundEvent(compare,decibel_value,cb_wrapper)

    # 开始分贝识别
    @_utils.check_type
    @staticmethod
    def start_decibel_recognition():
        def cb_wrapper(err,data):
            if err is None:
                if data and data.get('data') and data['data'] != '':
                    Auditory._decibel_val = data['data']
            else:
                _utils.raise_error('start_decibel_recognition',err,data)

        _subscribe_event.onSensorSoundEvent('==','',cb_wrapper)

    # 结束分贝识别
    @_utils.check_type
    @staticmethod
    def stop_decibel_recognition():
        _interface.call_api_async('web-ide', 'api.openDecibelDetectionPage', [{'type':'end'}])


class Visual:
    _sensors = {}

    # 将传感器绑定到对象的挂点
    @_utils.check_type
    @staticmethod
    def add_sensor( sensor: str, runtime_id: Union[int,float], attachment_id: tuple):
        attach=_utils.Handle_point(attachment_id)
        Visual._sensors[sensor] = (runtime_id, attach)

    # 获取传感器信息
    @_utils.check_type
    @staticmethod
    def get_sensor( sensor: str):
        if sensor in Visual._sensors:
            return Visual._sensors[sensor]
        else:
            raise KeyError(f"Sensor '{sensor}' not found")

    # 打开或关闭传感器画面
    @_utils.check_type
    @staticmethod
    def open_visual_sensor( sensor: str,action_type: bool=True):
        sensor_info=Visual.get_sensor(sensor)
        if action_type:
            func_name = 'unity.sensor.openVision'
            _interface.call_api('unity',func_name,[sensor_info[0], sensor_info[1],sensor])
        else:
            func_name = 'unity.sensor.closeVision'
            _interface.call_api('unity',func_name,[sensor])

class Temperature:

    _sensors = {}

    @_utils.check_type
    @staticmethod
    def add_sensor(sensor: str, runtime_id: Union[int,float]) ->None:
        Temperature._sensors[sensor] = runtime_id
        _interface.call_api('unity','unity.sensor.attachTemperature',[runtime_id])

    @_utils.check_type
    @staticmethod
    def get_sensor(sensor: str) -> int:
        if sensor in Temperature._sensors:
            return Temperature._sensors[sensor]
        else:
            raise KeyError(f"Sensor '{sensor}' not found")

    @_utils.check_type
    @staticmethod
    def onSensorTemperatureEvent(sensor:str, compare:str, temperature:Union[int,float],cb:Any):
        temperature = max(-40, min(temperature, 120))
        runtime_id = Temperature.get_sensor(sensor)
        def cb_wrapper(err,data):
            if err is None:
                cb()
            else:
                _utils.raise_error('onSensorTemperatureEvent',err,data)
        _subscribe_event.onSensorTemperatureEvent(runtime_id,compare,temperature,cb_wrapper)
    
    # 设置判定区域温度
    @_utils.check_type
    @staticmethod
    def set_temperature( area_id: Union[int,float], temp_val: Union[int,float]=0):
        temp_val = max(-40, min(temp_val, 120))
        _interface.call_api('unity','unity.sensor.setTemperature',[area_id,temp_val])

    # 持续检测判定区域温度
    @_utils.check_type
    @staticmethod
    def startTemperatureDetection(area_id:Union[int,float]):
        _subscribe_event.startTemperatureDetection(area_id)

    # 获取温度值
    @_utils.check_type
    @staticmethod
    def get_temperature_value( sensor: str):
        time.sleep(_constants.SLEEP_TIME)
        runtime_id=Temperature.get_sensor(sensor)
        temperature_value=_interface.call_api('unity','unity.sensor.getTemperature',[runtime_id])
        return temperature_value



class Humidity:

    _sensors = {}

    @_utils.check_type
    @staticmethod
    def add_sensor(sensor: str, runtime_id: Union[int,float]) ->None:
        Humidity._sensors[sensor] = runtime_id
        _interface.call_api('unity','unity.sensor.attachHumidity',[runtime_id])

    @_utils.check_type
    @staticmethod
    def get_sensor(sensor: str) -> int:
        if sensor in Humidity._sensors:
            return Humidity._sensors[sensor]
        else:
            raise KeyError(f"Sensor '{sensor}' not found")

    @_utils.check_type
    @staticmethod
    def onSensorHumidityEvent(sensor:str, compare:str, humidity_value:Union[int,float],cb:Any):
        humidity_value = int(humidity_value)
        humidity_value= max(0, min(humidity_value, 100))
        runtime_id = Humidity.get_sensor(sensor) 
        def cb_wrapper(err,data):
            if err is None:
                cb()
            else:
                _utils.raise_error('onSensorHumidityEvent',err,data)
        _subscribe_event.onSensorHumidityEvent(runtime_id,compare,humidity_value,cb_wrapper)

    # 设置判定区域湿度
    @_utils.check_type
    @staticmethod
    def set_humidity( area_id: Union[int,float], humidity_value: Union[int,float]=0):
        humidity_value = int(humidity_value)
        humidity_value= max(0, min(humidity_value, 100))
        _interface.call_api('unity','unity.sensor.setHumidity',[area_id,humidity_value])

    @_utils.check_type
    @staticmethod
    def get_humidity_value(senser:str):
        time.sleep(_constants.SLEEP_TIME)
        runtime_id=Humidity.get_sensor(senser)
        result= _interface.call_api('unity','unity.sensor.getHumidity',[runtime_id])
        return result

    # 持续检测判定区域湿度
    @_utils.check_type
    @staticmethod
    def startHumidityDetection( area_id: Union[int,float]):
        _subscribe_event.startHumidityDetection(area_id)


class Gravity:

    _sensors = {}

    @_utils.check_type
    @staticmethod
    def add_sensor(sensor: str, area_id: Union[int,float]) ->None:
        Gravity._sensors[sensor] = area_id
        _interface.call_api('unity','unity.sensor.attachGravity',[area_id])

    @_utils.check_type
    @staticmethod
    def get_sensor(sensor: str) -> int:
        if sensor in Gravity._sensors:
            return Gravity._sensors[sensor]
        else:
            raise KeyError(f"Sensor '{sensor}' not found")

    @_utils.check_type
    @staticmethod
    def onSensorGravityEvent(sensor: str, compare: str, gravity_value: Union[int,float],cb:Any):
        gravity_value = int(gravity_value)
        gravity_value= max(0, min(gravity_value, 10000))
        runtime_id = Gravity.get_sensor(sensor)
        def cb_wrapper(err,data):
            if err is None:
                cb()
            else:
                _utils.raise_error('onSensorGravityEvent',err,data)
        _subscribe_event.onSensorGravityEvent(runtime_id,compare,gravity_value,cb_wrapper)

    # 设置对象重力
    @_utils.check_type
    @staticmethod
    def set_gravity( runtime_id: Union[int,float], gravity_value: Union[int,float]=0):
        gravity_value = int(gravity_value)
        gravity_value= max(0, min(gravity_value, 10000))
        _interface.call_api('unity','unity.sensor.setGravity',[runtime_id,gravity_value])

    # 获取重力值
    @_utils.check_type
    @staticmethod
    def get_gravity_value( sensor: str):
        time.sleep(_constants.SLEEP_TIME)
        runtime_id = Gravity.get_sensor(sensor) 
        value= _interface.call_api('unity','unity.sensor.getGravity',[runtime_id])
        return value




import os
import logging
import datetime
from logging.handlers import TimedRotatingFileHandler

global_user_log_level = logging.ERROR
global_log_dir = None

def enable_user_log():
    global global_user_log_level
    global_user_log_level = logging.INFO

def disable_user_log():
    global global_user_log_level
    global_user_log_level = logging.ERROR

def set_log_dir(log_dir: str):
    global global_log_dir
    global_log_dir = log_dir

# ANSI escape codes for coloring output
class CustomFormatter(logging.Formatter):
    grey = "\x1b[38;5;245m"
    green = "\x1b[32m"
    yellow = "\x1b[33m"
    red = "\x1b[31m"
    bold_red = "\x1b[31;1m"
    blue = "\x1b[34m"
    white = "\x1b[37m"
    reset = "\x1b[0m"
    
    # 格式中的基本部分
    base_format = "%(asctime)s - %(levelname)s - "
    file_info_format = blue + "(%(filename)s:%(lineno)d)" + reset

    # 特定于日志级别的消息格式
    FORMATS = {
        logging.DEBUG: grey + base_format + grey + "%(message)s " + reset + file_info_format,
        logging.INFO: green + base_format + white + "%(message)s " + reset + file_info_format,
        logging.WARNING: yellow + base_format + yellow + "%(message)s " + reset + file_info_format,
        logging.ERROR: red + base_format + red + "%(message)s " + reset + file_info_format,
        logging.CRITICAL: bold_red + base_format + bold_red + "%(message)s " + reset + file_info_format
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt, "%Y-%m-%d %H:%M:%S")
        return formatter.format(record)

def setup_logger():
    global global_log_dir
    if global_log_dir:
        if not os.path.exists(global_log_dir):
            os.makedirs(global_log_dir)

        # 确定当前日期并设置初始文件名
        today = datetime.datetime.now().strftime("%Y-%m-%d")
        initial_log_filename = os.path.join(global_log_dir, f"crealand-sdk-{today}.log")

        # 检查文件是否存在，如果不存在，创建一个带日期后缀的文件
        if not os.path.exists(initial_log_filename):
            open(initial_log_filename, 'a').close()

    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    if (not logger.hasHandlers()):
        if global_log_dir:
            # 配置日志文件和TimedRotatingFileHandler
            file_handler = TimedRotatingFileHandler(initial_log_filename, when="midnight", interval=1)
            file_handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)'))
            logger.addHandler(file_handler)

        # 添加颜色日志到控制台
        global global_user_log_level
        console_handler = logging.StreamHandler()
        console_handler.setLevel(global_user_log_level)
        console_handler.setFormatter(CustomFormatter())
        logger.addHandler(console_handler)

    return logger



from crealand._core.bridge import _interface
from typing import Union
from crealand._utils import _utils

class Sound:
    # 播放声音
    @_utils.check_type
    @staticmethod
    def play(runtime_id: Union[int,float], res_id: str,is_loop: bool=False):
        _interface.call_api_async(
            'unity',
            "unity.sound.playSound",
            [runtime_id, is_loop, res_id],
        )

    # 声音
    @staticmethod
    @_utils.check_type
    def adjust_volume(runtime_id: Union[int,float], volume: int = 50):
        _interface.call_api_async(
            'unity',
            "unity.sound.adjustVolume",
            [runtime_id, max(0, min(volume, 100))],
        )

    # 停止播放
    @_utils.check_type
    @staticmethod
    def stop(runtime_id: Union[int,float]):
        _interface.call_api_async('unity', "unity.sound.stopSound", [runtime_id])

    # 设置背景音效
    @_utils.check_type
    @staticmethod
    def play_bgm(res_id: str):
        _interface.call_api_async('unity', "unity.sound.playBgSound", [res_id])

    # 背景音效音量
    @_utils.check_type
    @staticmethod
    def adjust_bgm_volume(volume: int = 50):
        _interface.call_api_async(
            'unity',
            "unity.sound.adjustBgVolume",
            [max(0, min(volume, 100))],
        )

    # 停止背景音效
    @staticmethod
    def stop_bgm():
        _interface.call_api_async(
            'unity',
            "unity.sound.stopBgSound",
            [],
        )


class Video:
    # 播放视频
    @_utils.check_type
    @staticmethod
    def play(res_id: str):
        _interface.call_api(
            'web-ide',
            "api.playVideo",
            [{'pythonOssId':res_id}],
        )


class Image:
    # 播放图片
    @_utils.check_type
    @staticmethod
    def show(res_id: str):
        _interface.call_api(
            'web-ide',
            "api.showImage",
            [{'pythonOssId':res_id}],
        )



import asyncio
import websockets
import json
import threading
import uuid
import atexit
from crealand._utils._logger_setup import setup_logger

logger = setup_logger()

global_event_loop = None
global_ws_client = None
global_listen_task = None
global_callback = None

class Callback:
    def __init__(self):
        self.callback_dict = {}
        self.broadcast_dict = {}

    def registerCallback(self, callback_id, callback):
        self.callback_dict[callback_id] = callback

    def registerBroadcast(self, info, callback):
        if info not in self.broadcast_dict:
            self.broadcast_dict[info] = []
            
        self.broadcast_dict[info].append(callback)

    def trigger(self, callback_id, err, data):
        if callback_id in self.callback_dict:
            logger.info(f'Find the callback: {callback_id}')
            self.callback_dict[callback_id](err, data)

    def broadcast(self, info):
        if info in self.broadcast_dict:
            logger.info(f'Find the callback: {info}')
            for callback in self.broadcast_dict[info]:
                callback()

def init_callback():
    global global_callback
    if global_callback is None:
        global_callback = Callback()
    return global_callback

def get_callback():
    global global_callback
    return global_callback

# 会话管理器
class WebSocketClient:
    def __init__(self, server_url: str, session_id: str, task_id: str, event_list: list):
        self.server_url = server_url
        self.session_id = session_id
        self.task_id = task_id
        self.event_list = event_list
        self.subscribe_event_msg_id_list = [None for _ in event_list]
        self.failed_event_list = []
        self.websocket = None
        self.connect_lock = threading.Lock()
        self.connected = False
        self.reconnect_interval = 5  # 重连间隔（秒）
        self.condition = asyncio.Event()
        self.init_index = True
        self.sync_msg_response = {}
        self.sync_msg_condition = {}

    async def connect(self):
        with self.connect_lock:
            if self.connected:
                return
            self.websocket = await websockets.connect(self.server_url)
            self.connected = True
            logger.info(f'Connected to {self.server_url}')
            for i in range(len(self.event_list)):
                msg_id = await self.subscribeEvent(self.event_list[i])
                self.subscribe_event_msg_id_list[i] = msg_id
                
    async def sendMessage(self, method: str, params=None):
        logger.info('Start to send message.')
        if not self.websocket or not self.connected:
            logger.error('WebSocket connection closed, attempting to reconnect...')
            self.connected = False
            await self.reconnect()
            
        msg_id = str(uuid.uuid4())
        message = json.dumps({
            'id': msg_id,
            'method': method, 
            'params': [params] if params else [], 
            'jsonrpc':'2.0'
        })
        await self.websocket.send(message)
        logger.info(f'Sent message: {message}')
        if method == 'crealand-api-transfer':
            return params['bridgeMsg']['id']
        else:
            return msg_id

    async def subscribeEvent(self, event_name: str):
        logger.info(f'Start to subscribe the event {event_name}.')
        return await self.sendMessage('rpc.on', event_name)

    async def sendToCrealandApiTransferAsync(self, dest: str, session_id: str, bridge_msg: dict):
        logger.info('Send message to crealand-api-transfer asynchronously.')
        params = {
            'method': 'call',
            'src' : 'sdk',
            'dest' : dest,
            'bridgeMsg': bridge_msg,
            'sessionID': session_id
        }
        if await self.sendMessage('crealand-api-transfer', params):
            return True
        else:
            return False

    async def sendToCrealandApiTransfer(self, dest: str, session_id: str, bridge_msg: dict):
        logger.info('Send message to crealand-api-transfer synchronously.')
        params = {
            'method': 'call',
            'src' : 'sdk',
            'dest' : dest,
            'bridgeMsg': bridge_msg,
            'sessionID': session_id
        }
        msg_id = await self.sendMessage('crealand-api-transfer', params)
        self.sync_msg_response[msg_id] = None
        self.sync_msg_condition[msg_id] = asyncio.Event()
        await self.sync_msg_condition[msg_id].wait()
        del self.sync_msg_condition[msg_id]
        response = self.sync_msg_response[msg_id]
        del self.sync_msg_response[msg_id]
        return response

    async def listen(self):
        try:
            msg_response_id = None
            msg_response_err = None
            msg_response_data = None
            async for message in self.websocket:
                data = json.loads(message)
                logger.info(f'Received message: {data}')
                if 'id' in data:
                    if data['id'] in self.subscribe_event_msg_id_list:
                        index = self.subscribe_event_msg_id_list.index(data['id'])
                        event_name = self.event_list[index]
                        if 'result' in data:
                            if event_name in data['result'] and data['result'][event_name] == 'ok':
                                logger.info(f'Subscribed to the event {event_name}')
                                if event_name in self.failed_event_list:
                                    self.failed_event_list.remove(event_name)
                            else:
                                logger.info(f'Fail to subscribe the event {event_name}')
                                self.failed_event_list.append(event_name)
                                await asyncio.sleep(self.reconnect_interval)
                                msg_id = await self.subscribeEvent(event_name)
                                self.subscribe_event_msg_id_list[index] = msg_id
                        else:
                            logger.info(f'Fail to subscribe the event {event_name}')
                            self.failed_event_list.append(event_name)
                            await asyncio.sleep(self.reconnect_interval)
                            msg_id = await self.subscribeEvent(event_name)
                            self.subscribe_event_msg_id_list[index] = msg_id
                elif 'notification' in data:
                    if data['notification'] in self.event_list:
                        if ('params' in data and len(data['params']) > 0 
                            and 'bridgeMsg' in data['params'][0] 
                            and 'id' in data['params'][0]['bridgeMsg']
                            and 'code' in data['params'][0]['bridgeMsg']):
                            if data['params'][0]['bridgeMsg']['code'] == 0:
                                if 'data' in data['params'][0]['bridgeMsg']:
                                    #logger.info(f'Received event {data["notification"]} params: {data["params"][0]["bridgeMsg"]["id"]}')
                                    msg_response_id = data['params'][0]['bridgeMsg']['id']
                                    msg_response_err = None
                                    msg_response_data = data['params'][0]['bridgeMsg']['data']
                            else:
                                if 'msg' in data['params'][0]['bridgeMsg']:
                                    msg_response_id = data['params'][0]['bridgeMsg']['id']
                                    msg_response_err = data['params'][0]['bridgeMsg']['code']
                                    msg_response_data = data['params'][0]['bridgeMsg']['msg']
                        elif (msg_response_id and 'params' in data and len(data['params']) > 0
                            and 'msgId' in data['params'][0] and 'msgEnd' in data['params'][0]
                            and data['params'][0]['msgId'] == msg_response_id
                            and data['params'][0]['msgEnd']):
                                logger.info(f'message id: {msg_response_id}')
                                logger.info('Start to trigger the callback')
                                api_callback = get_callback()
                                threading.Thread(
                                    target=api_callback.trigger, 
                                    args=(
                                        msg_response_id, 
                                        msg_response_err,
                                        msg_response_data, 
                                    )
                                ).start()
                                if (msg_response_id in self.sync_msg_response 
                                    and msg_response_id in self.sync_msg_condition):
                                    self.sync_msg_response[msg_response_id] = msg_response_data
                                    self.sync_msg_condition[msg_response_id].set()

                                msg_response_id = None
                                msg_response_err = None
                                msg_response_data = None

                if self.init_index and len(self.failed_event_list) == 0:
                    self.init_index = False
                    self.condition.set()
        except websockets.ConnectionClosed:
            logger.error('WebSocket connection closed, attempting to reconnect...')
            self.connected = False
            await self.reconnect()
        except Exception as e:
            logger.error(f'An error occurred: {e}')
        
    async def reconnect(self):
        while not self.connected:
            with self.connect_lock:
                if not self.connected:
                    await asyncio.sleep(self.reconnect_interval)
                    await self.connect()

    async def close(self):
        if self.websocket:
            await self.websocket.close()
            self.connected = False
            logger.info('WebSocket connection closed')

def init_event_loop():
    global global_event_loop
    if global_event_loop is None:
        global_event_loop = asyncio.new_event_loop()
        asyncio.set_event_loop(global_event_loop)
    return global_event_loop

def get_event_loop():
    global global_event_loop
    return global_event_loop

def get_ws_client():
    global global_ws_client
    return global_ws_client

def get_listen_task():
    global global_listen_task
    return global_listen_task

def exit_thread():
    logger.info('Start to execute the exit function.')
    task = get_listen_task()
    if task:
        task.cancel()
    loop = get_event_loop()
    if loop:
        ws_client = get_ws_client()
        task = asyncio.run_coroutine_threadsafe(ws_client.close(), loop)
        task.result()
        loop.call_soon_threadsafe(loop.stop)
        while True:
            if not loop.is_running():
                loop.close()
                break

    logger.info('Exit function has finished.')

async def ws_connect_async(server_url, session_id, task_id):
    global global_ws_client
    event_list = ['crealand-event-oncall-sdk']
    global_ws_client = WebSocketClient(server_url, session_id, task_id, event_list)
    # 尝试连接并订阅事件
    await global_ws_client.connect()
    # 在后台监听事件
    global global_listen_task
    global_listen_task = asyncio.create_task(global_ws_client.listen())
    # 阻塞直到订阅事件成功
    await global_ws_client.condition.wait()
    global_ws_client.condition = None

def ws_connect(server_url, session_id, task_id):
    try:
        # 注册退出执行程序
        atexit.register(exit_thread)
        # 获取全局事件循环
        loop = init_event_loop()
        loop.run_until_complete(ws_connect_async(server_url, session_id, task_id))
        loop.run_forever()
    except Exception as e:
        logger.error(f'An error occurred: {e}')

